#include <glib.h>
#include "util.h"

static void
dump_mem_line (gchar *prefix_str, const guint8 * mem, gsize mem_offset, gsize mem_size)
{
  gchar hexstr[50], ascstr[18], digitstr[4];

  if (mem_size > 16)
    mem_size = 16;

  hexstr[0] = '\0';
  ascstr[0] = '\0';

  if (mem != NULL) {
    guint i = 0;

    mem += mem_offset;
    while (i < mem_size) {
      ascstr[i] = (g_ascii_isprint (mem[i])) ? mem[i] : '.';
      g_snprintf (digitstr, sizeof (digitstr), "%02x ", mem[i]);
      g_strlcat (hexstr, digitstr, sizeof (hexstr));
      ++i;
    }
    ascstr[i] = '\0';
  }

  g_print ("%s%08x: %-48.48s %-16.16s\n",
      prefix_str, (guint) mem_offset, hexstr, ascstr);
}

void
dump_bytes (guint8 * data, guint length, guint indent_lvl)
{
  guint8 *prefix_str = g_malloc (indent_lvl + 1);
  guint buf_size = indent_lvl + 128;
  gchar *buf = g_malloc0 (buf_size);
  guint off = 0;

  memset (prefix_str, ' ', indent_lvl);
  prefix_str[indent_lvl] = '\0';

  while (off < length) {
    /* dump_mem_line will process 16 bytes at most */
    dump_mem_line (prefix_str, data, off, length - off);
    off += 16;
  }

  g_free (buf);
  g_free (prefix_str);
}
